/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.petra.sql.dsl;

import com.liferay.petra.sql.dsl.expression.ColumnAlias;
import com.liferay.petra.sql.dsl.expression.Expression;

/**
 * @author Preston Crary
 */
public interface Column<T extends Table<T>, C> extends Expression<C> {

	public static int FLAG_DEFAULT = 0;

	public static int FLAG_NULLITY = 1;

	public static int FLAG_PRIMARY = 2;

	@Override
	public ColumnAlias<T, C> as(String name);

	public int getFlags();

	public Class<C> getJavaType();

	public String getName();

	public int getSQLType();

	public T getTable();

	public default boolean isNullAllowed() {
		if ((getFlags() & FLAG_NULLITY) == 0) {
			return true;
		}

		return false;
	}

	public default boolean isPrimaryKey() {
		if ((getFlags() & FLAG_PRIMARY) == 0) {
			return false;
		}

		return true;
	}

}