/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.petra.sql.dsl;

import com.liferay.petra.sql.dsl.expression.Expression;
import com.liferay.petra.sql.dsl.factory.DSLQueryFactory;
import com.liferay.petra.sql.dsl.query.FromStep;

import java.util.Iterator;
import java.util.ServiceLoader;

/**
 * @author Preston Crary
 */
public class DSLQueryFactoryUtil {

	public static FromStep count() {
		return _DSL_QUERY_FACTORY.count();
	}

	public static FromStep countDistinct(Expression<?> expression) {
		return _DSL_QUERY_FACTORY.countDistinct(expression);
	}

	public static FromStep select() {
		return _DSL_QUERY_FACTORY.select();
	}

	public static FromStep select(Expression<?>... expressions) {
		return _DSL_QUERY_FACTORY.select(expressions);
	}

	public static <T extends Table<T>> FromStep select(T table) {
		return _DSL_QUERY_FACTORY.select(table);
	}

	public static FromStep selectDistinct(Expression<?>... expressions) {
		return _DSL_QUERY_FACTORY.selectDistinct(expressions);
	}

	public static <T extends Table<T>> FromStep selectDistinct(T table) {
		return _DSL_QUERY_FACTORY.selectDistinct(table);
	}

	private static final DSLQueryFactory _DSL_QUERY_FACTORY;

	static {
		ServiceLoader<DSLQueryFactory> serviceLoader = ServiceLoader.load(
			DSLQueryFactory.class, DSLQueryFactory.class.getClassLoader());

		Iterator<DSLQueryFactory> iterator = serviceLoader.iterator();

		_DSL_QUERY_FACTORY = iterator.next();
	}

}