/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.petra.sql.dsl.expression;

import com.liferay.petra.sql.dsl.ast.ASTNode;
import com.liferay.petra.sql.dsl.query.DSLQuery;
import com.liferay.petra.sql.dsl.query.sort.OrderByExpression;

/**
 * @author Preston Crary
 */
public interface Expression<T> extends ASTNode {

	public Alias<T> as(String name);

	public OrderByExpression ascending();

	public OrderByExpression descending();

	public Predicate eq(Expression<T> expression);

	public Predicate eq(T value);

	public Predicate gt(Expression<T> expression);

	public Predicate gt(T value);

	public Predicate gte(Expression<T> expression);

	public Predicate gte(T value);

	public Predicate in(DSLQuery dslQuery);

	public Predicate in(T[] values);

	public Predicate isNotNull();

	public Predicate isNull();

	public Predicate like(Expression<String> expression);

	public Predicate like(String value);

	public Predicate lt(Expression<T> expression);

	public Predicate lt(T value);

	public Predicate lte(Expression<T> expression);

	public Predicate lte(T value);

	public Predicate neq(Expression<T> expression);

	public Predicate neq(T value);

	public Predicate notIn(DSLQuery dslQuery);

	public Predicate notIn(T[] values);

	public Predicate notLike(Expression<String> expression);

	public Predicate notLike(String value);

}